import { useEffect, useRef, useState } from 'react'
import * as THREE from 'three';
import InputWithDropDown from '../../../../../ui/inputs/InputWithDropDown'
import RenameInput from '../../../../../ui/inputs/RenameInput'
import LabledDropdown from '../../../../../ui/inputs/LabledDropdown'
import Trigger from '../trigger/Trigger'
import { useSelectedEventData, useSelectedProduct } from "../../../../../../store/simulation/useSimulationStore";
import { useProductStore } from "../../../../../../store/simulation/useProductStore";
import { AddIcon, RemoveIcon, ResizeHeightIcon } from '../../../../../icons/ExportCommonIcons'
import { handleResize } from '../../../../../../functions/handleResizePannel'
import PickAndPlaceAction from '../actions/PickAndPlaceAction'

function RoboticArmMechanics() {
    const actionsContainerRef = useRef<HTMLDivElement>(null);

    const [activeOption, setActiveOption] = useState("pickAndPlace");
    const [selectedItem, setSelectedItem] = useState<{ item: { uuid: string; name: string } | null; }>({ item: null });
    const { selectedEventData } = useSelectedEventData();
    const { getEventByModelUuid, addAction } = useProductStore();
    const { selectedProduct } = useSelectedProduct();

    const availableActions = {
        defaultOption: "pickAndPlace",
        options: ["pickAndPlace"]
    };
    const [actions, setActions] = useState<{ uuid: string; name: string }[]>([]);

    useEffect(() => {
        const event = getCurrentEventData();
        const actionList = getActionList(event as RoboticArmEventSchema);
        setActions(actionList);

        if (actionList.length > 0 && !selectedItem.item) {
            setSelectedItem({ item: actionList[0] });
        }
    }, [selectedEventData, selectedProduct]);

    const getCurrentEventData = () => {
        if (!selectedEventData?.data || !selectedProduct) return null;
        return getEventByModelUuid(selectedProduct.productId, selectedEventData.data.modelUuid) || null;
    };

    const getActionList = (event: RoboticArmEventSchema) => {
        if (!event || !('point' in event)) return [];

        return event.point.actions.flatMap(
            (action) =>
                action.triggers.map((trigger) => ({
                    uuid: trigger.triggerUuid,
                    name: trigger.triggerName,
                }))
        ) || [];
    };

    const handleActionToggle = (actionUuid: string) => {
        const action = actions.find(a => a.uuid === actionUuid);
        if (action) {
            setSelectedItem({ item: action });
        }
    };

    const handleAddAction = () => {
        if (selectedEventData) {
            const newEvent = {
                actionUuid: THREE.MathUtils.generateUUID(),
                actionName: `Action ${actions.length + 1}`,
                actionType: "pickAndPlace" as const,
                process: {
                    startPoint: null as [number, number, number] | null,
                    endPoint: null as [number, number, number] | null
                },
                triggers: [] as TriggerSchema[]
            }
            addAction(
                selectedProduct.productId,
                selectedEventData?.data.modelUuid,
                selectedEventData?.selectedPoint,
                newEvent
            )
        }
    };

    const handleDeleteAction = (actionUuid: string) => {

    };

    return (
        <>
            <div className="global-props">
                <div className="property-list-container">
                    <div className="property-item">
                        <InputWithDropDown
                            label="Speed"
                            value="0.5"
                            min={0}
                            step={0.1}
                            defaultValue={'0.5'}
                            max={10}
                            activeOption="s"
                            onClick={() => { }}
                            onChange={(value) => console.log(value)}
                        />
                    </div>
                </div>
            </div>
            <div className="actions-list-container">
                <div className="actions">
                    <div className="header">
                        <div className="header-value">Actions</div>
                        <div className="add-button" onClick={() => handleAddAction()}>
                            <AddIcon /> Add
                        </div>
                    </div>
                    <div
                        className="lists-main-container"
                        ref={actionsContainerRef}
                        style={{ height: "120px" }}
                    >
                        <div className="list-container">
                            {actions.map((action) => (
                                <div
                                    key={action.uuid}
                                    className={`list-item ${selectedItem.item?.uuid === action.uuid ? "active" : ""}`}
                                >
                                    <div
                                        className="value"
                                        onClick={() => handleActionToggle(action.uuid)}
                                    >
                                        <RenameInput value={action.name} />
                                    </div>
                                    {actions.length > 1 && (
                                        <div
                                            className="remove-button"
                                            onClick={() => handleDeleteAction(action.uuid)}
                                        >
                                            <RemoveIcon />
                                        </div>
                                    )}
                                </div>
                            ))}
                        </div>
                        <div
                            className="resize-icon"
                            id="action-resize"
                            onMouseDown={(e) => handleResize(e, actionsContainerRef)}
                        >
                            <ResizeHeightIcon />
                        </div>
                    </div>
                </div>
            </div>
            <div className="selected-actions-details">
                <div className="selected-actions-header">
                    <RenameInput value={selectedItem.item?.name || "Action Name"} />
                </div>
                <div className="selected-actions-list">
                    <LabledDropdown
                        defaultOption={availableActions.defaultOption}
                        options={availableActions.options}
                        onSelect={(option) => setActiveOption(option)}
                    />
                    {activeOption === "pickAndPlace" && <PickAndPlaceAction />}
                </div>
            </div>
            <div className="tirgger">
                <Trigger />
            </div>
        </>
    )
}

export default RoboticArmMechanics